"""
Selectors for querying artist information
"""

from argparse import ArgumentParser
from functools import partial
from typing import Iterable

from spotify_model import Paging, SearchAlbum
from spotipy import Spotify

from .util import exhaust, read_credentials_server


def artist_albums(client: Spotify, artist_ids: Iterable[str]) -> Iterable[SearchAlbum]:
    """
    https://developer.spotify.com/documentation/web-api/reference/#category-artists
    """

    def _artist_albums(artist_id: str, limit: int, offset: int) -> Paging:
        return Paging(**client.artist_albums(artist_id, limit=limit, offset=offset))

    for artist_id in artist_ids:
        albums_function = partial(_artist_albums, artist_id)

        for album in exhaust(albums_function):
            yield SearchAlbum(**album)


def main() -> None:
    "Simple runner for quickly retrieving artist info"
    parser = ArgumentParser()
    parser.add_argument("-c", "--credentials", required=True)
    parser.add_argument("artist_ids", nargs="+")
    cmdline = parser.parse_args()

    client = read_credentials_server(cmdline.credentials)

    print("Artist albums:")
    for album in artist_albums(client, cmdline.artist_ids):
        print(album)


if __name__ == "__main__":
    main()
